Package: splines2
Title: Regression Spline Functions and Classes
Version: 0.4.5
Authors@R: c(
    person(given = "Wenjie", family = "Wang",
           email = "wang@wwenjie.org",
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-0363-3180")),
    person(given = "Jun", family = "Yan", role = "aut",
           comment = c(ORCID = "0000-0003-4401-7296"))
    )
Description: Constructs basis matrix of B-splines, M-splines,
    I-splines, convex splines (C-splines), periodic M-splines,
    natural cubic splines, generalized Bernstein polynomials,
    and their integrals (except C-splines) and derivatives
    of given order by close-form recursive formulas.
    It also contains a C++ head-only library integrated with Rcpp.
    See Wang and Yan (2021) <doi:10.6339/21-JDS1020> for details.
Imports: Rcpp, stats
LinkingTo: Rcpp, RcppArmadillo
Suggests: knitr, rmarkdown, tinytest
Depends: R (>= 3.2.3)
VignetteBuilder: knitr
License: GPL (>= 3)
URL: https://wwenjie.org/splines2,
        https://github.com/wenjie2wang/splines2
BugReports: https://github.com/wenjie2wang/splines2/issues
Encoding: UTF-8
RoxygenNote: 7.1.1
NeedsCompilation: yes
Packaged: 2021-09-19 00:33:10 UTC; wenjie
Author: Wenjie Wang [aut, cre] (<https://orcid.org/0000-0003-0363-3180>),
  Jun Yan [aut] (<https://orcid.org/0000-0003-4401-7296>)
Maintainer: Wenjie Wang <wang@wwenjie.org>
Repository: CRAN
Date/Publication: 2021-09-19 13:20:02 UTC
Built: R 4.0.5; x86_64-w64-mingw32; 2021-09-19 17:00:29 UTC; windows
Archs: x64
