# Generated by roxygen2: do not edit by hand

S3method(deriv,bSpline2)
S3method(deriv,bernsteinPoly)
S3method(deriv,cSpline)
S3method(deriv,dbs)
S3method(deriv,iSpline)
S3method(deriv,ibs)
S3method(deriv,mSpline)
S3method(deriv,naturalSpline)
S3method(knots,bSpline2)
S3method(knots,bernsteinPoly)
S3method(knots,cSpline)
S3method(knots,dbs)
S3method(knots,iSpline)
S3method(knots,ibs)
S3method(knots,mSpline)
S3method(knots,naturalSpline)
S3method(makepredictcall,bSpline2)
S3method(makepredictcall,bernsteinPoly)
S3method(makepredictcall,cSpline)
S3method(makepredictcall,dbs)
S3method(makepredictcall,iSpline)
S3method(makepredictcall,ibs)
S3method(makepredictcall,mSpline)
S3method(makepredictcall,naturalSpline)
S3method(predict,bSpline2)
S3method(predict,bernsteinPoly)
S3method(predict,cSpline)
S3method(predict,dbs)
S3method(predict,iSpline)
S3method(predict,ibs)
S3method(predict,mSpline)
S3method(predict,naturalSpline)
S3method(print,bSpline2)
S3method(print,bernsteinPoly)
S3method(print,cSpline)
S3method(print,dbs)
S3method(print,iSpline)
S3method(print,ibs)
S3method(print,mSpline)
S3method(print,naturalSpline)
export(bSpline)
export(bernsteinPoly)
export(cSpline)
export(dbs)
export(iSpline)
export(ibs)
export(mSpline)
export(naturalSpline)
importFrom(Rcpp,sourceCpp)
importFrom(stats,deriv)
importFrom(stats,knots)
importFrom(stats,makepredictcall)
importFrom(stats,predict)
useDynLib(splines2)
